@@ -19,9 +19,7 @@
 # include <fcntl.h>
 #endif
 #include <sys/file.h>
-#ifdef	HAVE_SYSLOG
 #include <syslog.h>
-#endif	/* HAVE_SYSLOG */
 
 #define	SMTPSERVER	"smtpserver"
 #define	ROUTER		"router"
@@ -389,12 +387,9 @@ otherprog:
 		  n = errno;
 		  fprintf(stderr, "%s: cannot submit mail (err# %d)!\n",
 			  zmailer, errno);
-#ifdef	LOG_CRIT
-#ifdef	LOG_MAIL
+
 		  openlog("sendmail", LOG_PID, LOG_MAIL);
-#else  /* !LOG_MAIL */
-		  openlog("sendmail", LOG_PID);
-#endif /* LOG_MAIL */
+
 		  errno = n;
 		  syslog(
 #ifdef	LOG_SALERT
@@ -403,7 +398,6 @@ otherprog:
 			 LOG_ALERT
 #endif /* LOG_SALERT */
 			 , "cannot submit mail for uid %d: %m", uid);
-#endif /* LOG_CRIT */
 		  mfp = deadletter(uid);
 		}
 		if (truid != uid) {
