@@ -19,12 +19,10 @@ struct address {
 	char		**routermxes;	/* [mea] hostpointers from router */
 };
 
-#define _DSN_RET_FULL		0x001
-#define _DSN_RET_HDRS		0x002
-#define _DSN_NOTIFY_SUCCESS	0x004
-#define _DSN_NOTIFY_FAILURE	0x008
-#define _DSN_NOTIFY_DELAY	0x010
-#define _DSN_NOTIFY_NEVER	0x020
+#define _DSN_NOTIFY_SUCCESS	0x001
+#define _DSN_NOTIFY_FAILURE	0x002
+#define _DSN_NOTIFY_DELAY	0x004
+#define _DSN_NOTIFY_NEVER	0x008
 
 /* `convertmode' controls the behaviour of the message conversion:
      _CONVERT_NONE (0): send as is
@@ -43,6 +41,7 @@ struct rcpt {
 	struct address	*addr;		/* addr.link is the sender address */
 	char		*orcpt;		/* DSN ORCPT= string */
 	char		*notify;	/* DSN NOTIFY= flags */
+	int		notifyflgs;
 	char		***newmsgheader; /* message header line pointer ptr
 					   that points to an address of
 					      ctldesc->msgheaders[index]
@@ -144,9 +143,16 @@ extern int	getrrtype __((char *, int, int, int));
 extern int	       emptyline __((char *line, int size));
 
 /* esyslib.c: */
+extern int cistrcmp __((char *s1, char *s2));
+extern int efstat __((int fd, struct stat *stbuf));
+extern int emkdir __((char *dirpath, int mode));
 extern int eopen __((char *s, int f, int m));
-extern int eread __((int fd, char *buf, int len));
 extern int epipe __((int fdarr[2]));
+extern int eread __((int fd, char *buf, int len));
+extern int errno;
+extern int estat __((char *filename, struct stat *stbuf));
+extern int eunlink __((char *filename));
+extern int optind;
 extern int embytes;
 extern int emcalls;
 extern unsigned emsleeptime;
