@@ -720,8 +720,7 @@ _splay(n, q)
 
 #ifdef DEBUG
 	if (q->root != prev) {
-		/* fprintf(stderr, " *** bug in splay: n not in q *** "); */
-		abort();
+	  /* fprintf(stderr, " *** bug in splay: n not in q *** "); */ abort();
 	}
 #endif
 
