@@ -62,7 +62,7 @@ extern int errno;
 
 /* array of message file name associated with a file descriptor */
 static char **mail_file = NULL;
-static int mail_nfiles = 0;
+static int mail_nfiles  = 0;
 char *postoffice;	/* may be extern or local */
 
 /*
@@ -237,20 +237,20 @@ mail_open(type)
 	unlink(scratch);
 	mail_free(scratch);
 
-	/* Housekeeping */
+	/* Extend when need! */
 
-	if (mail_file == NULL) {
-		/* getdtablesize() is defined in ../libc/detach.c on systems
-		 * that don't have it. */
-		int nfile;
-		nfile = mail_nfiles = getdtablesize();	/* XX: assumed static */
-
-		mail_file = (char**)mail_alloc((u_int)((sizeof (char*))*nfile));
-		while (--nfile >= 0)
-			mail_file[nfile] = NULL;
+	if (FILENO(fp) > mail_nfiles) {
+	  int nfile = FILENO(fp)+1;
+	  if (mail_file == NULL) {
+	    mail_file = (char**)mail_alloc((u_int)(sizeof(char*) * nfile));
+	  } else {
+	    mail_file = (char**)mail_realloc((u_int)(mail_file,sizeof(char*) * nfile));
+	  }
+	  while (mail_nfiles < nfile) {
+	    mail_file[mail_nfiles] = NULL;
+	    ++mail_nfiles;
+	  }
 	}
-	if (FILENO(fp) >= mail_nfiles)
-		abort();
 	mail_file[FILENO(fp)] = message;
 
 	/* Grab preferences from the environment to initialize the envelope */
@@ -291,7 +291,7 @@ mail_abort(fp)
 		return -1;
 	}
 	if (FILENO(fp) >= mail_nfiles)
-		abort();
+		abort(); /* Usage error -- no such fileno in our use! */
 	messagep = &mail_file[FILENO(fp)];
 	if (*messagep == NULL) {
 		errno = ENOENT;
@@ -328,7 +328,7 @@ mail_close(fp)
 		return -1;
 	}
 	if (FILENO(fp) >= mail_nfiles)
-		abort();
+		abort(); /* Usage error -- no such fileno in our use! */
 	messagep = &mail_file[FILENO(fp)];
 	if (*messagep == NULL) {
 		errno = ENOENT;
@@ -460,7 +460,7 @@ mail_close_alternate(fp,where,suffix)
 		return -1;
 	}
 	if (FILENO(fp) >= mail_nfiles)
-		abort();
+		abort(); /* Usage error -- no such fileno in our use! */
 	messagep = &mail_file[FILENO(fp)];
 	if (*messagep == NULL) {
 		errno = ENOENT;
