@@ -42,7 +42,7 @@ char *sys_siglist[] = {
 #endif	/* HAVE_SYS_SIGLIST */
 
 #ifndef HAVE_STRSIGNAL
-#ifndef HAVE_EXTERN_STR_SIGLIST
+#ifndef HAVE_EXTERN_SYS_SIGLIST
 extern const char *sys_siglist[];
 #endif
 
