@@ -72,22 +72,7 @@ extern char *index(), *strcpy(), *strncpy(), *strcat();
 /* =()<extern @<SIZE_T>@	strlen();>()= */
 extern int	strlen();
 
-/*
-**  Return a string representation of errno.
-*/
-static char *
-xstrerror(e)
-    int		e;
-{
-    extern int	sys_nerr;
-    extern char	*sys_errlist[];
-    static char	buff[30];
-
-    if (e >= 0 && e < sys_nerr)
-	return sys_errlist[e];
-    (void)sprintf(buff, "Error code %d\n", e);
-    return buff;
-}
+extern char *strerror();
 
 void vsyslog(pri, fmt, ap)
 	int pri;
@@ -135,12 +120,11 @@ void vsyslog(pri, fmt, ap)
 	/* substitute error message for %m */
 	{
 		register char ch, *t1, *t2;
-		char *xstrerror();
 
 		for (t1 = fmt_cpy; ch = *fmt; ++fmt)
 			if (ch == '%' && fmt[1] == 'm') {
 				++fmt;
-				for (t2 = xstrerror(saved_errno);
+				for (t2 = strerror(saved_errno);
 				    *t1 = *t2++; ++t1);
 			}
 			else
@@ -184,7 +168,11 @@ void vsyslog(pri, fmt, ap)
 
 
 #ifdef HAVE_STDARG_H
-void syslog(int pri, char *fmt, ...)
+void syslog(int pri, char *fmt
+#ifdef __STDC__
+	    , ...
+#endif
+	    )
 #else
 /* VARARGS0 */
 void syslog(va_alist)
