@@ -128,15 +128,13 @@ whathost(file)
 		   Lets try to find the real one, if we can! */
 		continue;
 	      }
-	      free(mounts);
-	      return hostname;
 	    } else {
-	      free(mounts);
-	      return "localhost";
+	      strcpy(hostname,"localhost");
 	    }
+	    break;
 	  }
 	}
-	free(mounts);
+	/*free(mounts);*/ /* DONT FREE IT! */
 	if (*hostname != 0)
 	  return hostname;
 	return NULL;
