@@ -168,8 +168,12 @@ vfprintf(fp, fmt, ap)
 #endif	/* !HAVE_VPRINTF */
 
 int
-#ifdef HAVE_STDARG_H /* Also ANSI-C .. */
-siofprintf(FILE *fp, char *fmt, ...)
+#ifdef HAVE_STDARG_H
+siofprintf(FILE *fp, char *fmt
+# ifdef __STDC__
+	   , ...
+# endif
+	   )
 #else
 siofprintf(va_alist)
 	va_dcl
@@ -197,7 +201,11 @@ siofprintf(va_alist)
 
 int
 #ifdef HAVE_STDARG_H
-sioprintf(char *fmt, ...)
+sioprintf(char *fmt
+# ifdef __STDC__
+	  , ...
+# endif
+	  )
 #else
 sioprintf(va_alist)
 	va_dcl
