@@ -152,7 +152,7 @@ string(avp, i)
 {
 	int len;
 	
-	if (**avp == NULL) abort();
+	if (**avp == NULL) abort(); /* calling convention error! */
 	len = strlen(**avp);
 	++*avp;
 	return (i ? (!len) : len);
