@@ -194,7 +194,7 @@ extern int funclevel;
 extern int ci2strncmp();
 extern char *progname;
 extern int fprintToken();
-extern int printToken();
+extern int printdToken();
 
 #ifdef DEBUG
 extern int D_regnarrate, D_compare, D_matched;
@@ -1380,28 +1380,34 @@ tregsub(prog, n)
 	register struct token *ts, *te;
 	u_char *cp, *ep;
 	int len;
+	int buflen;
+	char *buf;
 
 	regprog = prog;
 	if (prog != NULL && UCHARAT(prog->program) != MAGIC)
 		FAIL("damaged tregexp fed to tregsub");
 
-	ts = prog->startp[n];
-	te = prog->endp[n];
-
 	if (n < 0 || n > 9)
 		FAIL("invalid substitution parameter requested");
 
-	if ((ts = prog->startp[n]) == NULL)
+	ts = prog->startp[n];
+	te = prog->endp[n];
+
+	if (ts == NULL)
 		/*FAIL("null substitution expansion");*/
 		return NULL;
 
-	for (len = 0; ts != te ; ts = ts->t_next)
-		len += fprintToken((FILE *)NULL, ts, 1);
 	ts = prog->startp[n];
-	cp = (u_char *)tmalloc(len+1);
-	ep = cp+(len+1);
-	ep = cp + printToken(cp, ep, ts, te, 0);
-	*ep = '\0';
+
+	buflen = 4000;
+	buf = malloc(buflen);
+	len = printdToken(&buf, &buflen, ts, te, 0);
+	buf[len] = '\0';
+
+	cp = tmalloc(len+1);
+	memcpy(cp,buf,len+1);
+	free(buf);
+
 	return cp;
 }
 #endif	/* MAILER */
