@@ -91,6 +91,7 @@ extern void initline();
 extern void setfreefd();
 extern void v_purge();
 
+extern char *newattribute_2 __((struct conscell *, char *, char*));
 
 #ifndef strchr
 extern char *strchr(), *strrchr();
@@ -1496,8 +1497,6 @@ run_listexpand(avl, il)
 
 #define use_lapply 1
 #ifdef use_lapply
-		l       = newstring((u_char*)strsave(buf));
-		cdr(l)  = conststring((u_char*)origaddr);
 		*DSNbuf = 0;
 		if (notary != NULL)
 		  strncpy(DSNbuf,notary,sizeof(DSNbuf)-30);
@@ -1523,10 +1522,11 @@ run_listexpand(avl, il)
 		  ++s;
 		}
 		*s2 = 0;
-		s = build_gensym(privilege,"recipient",DSNbuf,errors_to);
+		s = newattribute_2(attributes,"DSN",DSNbuf);
 
-		lrc = newstring((u_char*)strsave(s));
-		cddr(l) = lrc;
+		l       = newstring((u_char*)strsave(buf));
+		cdr(l)  = conststring((u_char*)origaddr);
+		cddr(l) = newstring((u_char*)strsave(s));
 		l = ncons(l);
 
 		deferit = 0;
@@ -2016,7 +2016,7 @@ run_runas(argc, argv)
 	r = s_apply(argc - 2, argv + 2);
 	if (myuid == 0 && uid != 0) {
 	  if (setreuid(0, 0) < 0)
-	    abort();
+	    abort(); /* user-identity change failed! */
 	}
 
 	return r;
