@@ -14,10 +14,10 @@
 #include <errno.h>
 #include "mail.h"
 #include "libz.h"
-#ifdef	HAVE_SYSLOG
-#include <syslog.h>
-#endif	/* HAVE_SYSLOG */
+#include "syslog.h"
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include "prototypes.h"
 
@@ -191,7 +191,7 @@ run_rfc822(argc, argv)
 		defer(e, "ctrlfile");
 		break;
 	default:
-		abort();
+		abort(); /* Impossible processing status */
 	}
 
 	if (D_final > 0)
@@ -1005,7 +1005,6 @@ struct rwmatrix {
 	struct rwmatrix	*next;
 	struct rwmatrix	*down;
 	struct conscell *info;	/* rewrite, sender, or recipient list info */
-	struct notary   *dsn;
 	struct conscell *errto;
 	union {
 		int	number;	/* XOR address id number for recipients */
@@ -1022,7 +1021,6 @@ rwalloc(rwpp)
 
 	p = (struct rwmatrix *)tmalloc(sizeof (struct rwmatrix));
 	p->down = NULL;
-	p->dsn  = NULL;
 	p->errto = NULL;
 	p->urw.number = 0;			/* unused but why not */
 	if (*rwpp != NULL)
@@ -1356,14 +1354,14 @@ sequencer(e, file)
 			    h = mkSender(e,uidpwnam(e->e_statbuf.st_uid),0);
 			}
 			if (h == NULL)
-				abort();
+				abort(); /* Failed to make sender header */
 			l = router(h->h_contents.a,
 				   e->e_statbuf.st_uid, "sender");
 			if (l == NULL) {
 			  /* From: <>,  and no envelope 'from' .. */
 			  h = mkSender(e,uidpwnam(e->e_statbuf.st_uid),0);
 			  if (h == NULL)
-			    abort();
+			    abort(); /* Can't make Sender header ?? */
 			  l = router(h->h_contents.a,
 				     e->e_statbuf.st_uid, "sender");
 			}
@@ -1638,7 +1636,8 @@ sequencer(e, file)
 		/* first level is ANDs */
 		++idnumber, nxor = 0;
 		for (to = car(l); to != NULL; to = cdr(to)) {
-			struct conscell *x, *tmp, *errto;
+			struct conscell *x, *tmp, *errto, *gg;
+			struct conscell *sender1;
 
 			/* secondlevel is XORs */
 #if 0
@@ -1659,26 +1658,34 @@ sequencer(e, file)
 #endif
 			++nxor;
 			tmp   = copycell(car(x));
-			errto = find_errto(cdar(x));
 			cdr(tmp) = NULL;
+			sender1 = cdar(x);
+			gg = cdr(cddr(cadr(sender1))); /* recipient attrib! */
+			errto = find_errto(gg);   /* Possible 'ERR' value */
+			if (!errto)
+			  errto = cddar(sender1);  /* Sender 'user' */
+
+			/* Rewriter level: */
 			for (rwp = rwhead; rwp != NULL; rwp = rwp->next)
-				if (s_equal(rwp->info, tmp) &&
-				    s_equal(rwp->errto,errto))
+				if (s_equal(rwp->info, tmp))
 					break;
 			if (rwp == NULL) {
 				rwp = rwalloc(&rwhead);
-				rwp->info = tmp;	/* rewritings */
-				rwp->errto = errto;
+				rwp->info  = tmp;	/* rewritings */
 			}
-			tmp = cddar(x);
-			cddar(x) = NULL;
+			/* else the 'tmp' leaks for a moment ? */
+			
+			tmp = cdr(sender1); /* recipient */
+			cdr(sender1) = NULL; /* detach from sender */
+
+			/* Sender (and errto) level: */
 			for (nsp = rwp->down; nsp != NULL; nsp = nsp->next)
-				if (s_equal(nsp->info, cdar(x)) &&
-				    s_equal(nsp->errto,errto))
+				if (s_equal(nsp->info, sender1) &&
+				    s_equal1(rwp->errto,errto))
 					break;
 			if (nsp == NULL) {
 				nsp = rwalloc(&rwp->down);
-				nsp->info = cdar(x);	/* new sender */
+				nsp->info  = sender1;	/* new sender */
 				nsp->errto = errto;
 			}
 #if 0
@@ -1694,7 +1701,7 @@ sequencer(e, file)
 			rcp = rwalloc(&nsp->down);
 			rcp->info = tmp;		/* new recipient */
 			rcp->urw.number = idnumber;
-			rcp->errto = NULL;
+			rcp->errto = errto;
 #if 0
 			for (rwp = rwhead; rwp != NULL; rwp = rwp->next) {
 				printf("**");
@@ -1879,15 +1886,20 @@ sequencer(e, file)
 
 	for (rwp = rwhead; rwp != NULL; rwp = rwp->next) {
 
-		if (!iserrmessage()  &&  rwp->errto != NULL) {
-			/* print envelope sender address */
-                        putc(_CF_ERRORADDR, ofp);
-                        putc(_CFTAG_NORMAL, ofp);
-                        fprintf(ofp,"%s\n",(char*)(rwp->errto));
-		}
-
 		for (nsp = rwp->down; nsp != NULL; nsp = nsp->next) {
 
+			if (!iserrmessage()  &&  nsp->errto != NULL) {
+				/* print envelope sender address */
+				putc(_CF_ERRORADDR, ofp);
+				putc(_CFTAG_NORMAL, ofp);
+				fprintf(ofp,"%s\n",nsp->errto->string);
+				if (vfp != NULL) {
+				  putc(_CF_ERRORADDR, vfp);
+				  putc(_CFTAG_NORMAL, vfp);
+				  fprintf(vfp,"%s\n",nsp->errto->string);
+				}
+			}
+
 			fprintf(ofp, "%c%c", _CF_SENDER, _CFTAG_NORMAL);
 			prctladdr(nsp->info, ofp, _CF_SENDER, "sender");
 			putc('\n', ofp);
@@ -2096,26 +2108,19 @@ find_errto(info)
 struct conscell *info;
 {
 	int i = 0;
-	register struct conscell *l, *x = NULL;
+	register struct conscell *x = NULL;
 
-	for (l = car(info); l != NULL; l = cdr(l)) {
-	  ++i;
-	  if (STRING(l)) {
-	    if (cdr(l) == NULL
-		&& (x = v_find(l->string)) != NULL
-		&& (x = cdr(x)) != NULL
-		&& LIST(x)) {
-	      for (x = car(x); x != NULL; x = cddr(x)) {
-		if (STRING(x) && strcmp((char *)x->string,"ERR") == 0) {
-		  x = cdr(x);
-		  break;
-		}
-	      }
-	      return x;
-	      /* if x == NULL, no Errors-To -expansion was specified */
-	    }
-	  } else
-	    return NULL; /* A list ??  Shall not be! */
+	if (STRING(info))
+	  info = v_find(info->string);
+	if (!info) return NULL;
+
+	for (x = cadr(info); x != NULL; x = cddr(x)) {
+	  if (!STRING(x))
+	    return NULL; /* error in data */
+	  if (strcmp((char *)x->string,"ERR") == 0) {
+	    x = cdr(x);
+	    break;
+	  }
 	}
 	return x;
 }
