@@ -20,9 +20,7 @@
 #include <fcntl.h>
 #include <sys/file.h>
 #include "mail.h"
-#ifdef	HAVE_SYSLOG
-#include <syslog.h>
-#endif	/* HAVE_SYSLOG */
+#include "syslog.h"
 #include "interpret.h"
 #include "splay.h"
 
@@ -280,13 +278,7 @@ main(argc, argv)
 		}
 		else
 		  router_id = getpid();
-#ifdef	LOG_PID
-#ifdef	LOG_MAIL
 		openlog("router", LOG_PID, LOG_MAIL);
-#else	/* !LOG_MAIL */
-		openlog("router", LOG_PID);
-#endif	/* LOG_MAIL */
-#endif	/* LOG_PID */
 	}
 	if (c < argc) {
 	  savefile = 1;
