@@ -15,9 +15,7 @@
 #include <fcntl.h>
 #include "mail.h"
 #include "scheduler.h"
-#ifdef	HAVE_SYSLOG_H
-#include <syslog.h>
-#endif	/* HAVE_SYSLOG_H */
+#include "syslog.h"
 #include "ta.h"
 
 #include "prototypes.h"
@@ -42,6 +40,7 @@ struct not {
 	char *not;
 	char *orcpt;
 	char *notify;
+	int   notifyflgs;
 	char *rcpt;
 	char *rcpntp;
 };
@@ -109,7 +108,7 @@ scnotaryreport(errfp,notary)
 	  typetag = type_local;
 	if (notary->orcpt)
 	  fprintf(errfp, "Original-Recipient: %s\n", notary->orcpt);
-	fprintf(errfp, "Final-Recipient: %s; %s\n", typetag, rcpt);
+	fprintf(errfp, "Final-Recipient: %s;%s\n", typetag, rcpt);
 	fprintf(errfp, "Action: %s\n", action);
 	if (status)
 	  fprintf(errfp, "Status: %s\n", status);
@@ -220,9 +219,7 @@ reporterrs(cfpi)
 		return;
 	if (cfpi->erroraddr == NULL)
 		return;
-	if (cfpi->errorchan != NULL &&
-	    strcmp(cfpi->errorchan,"error") == 0)
-		no_error_report = 1;
+	no_error_report = cfpi->iserrmesg;
 
 	boundarystr = (char*)buildboundary("dlvryerr");
 
@@ -315,6 +312,7 @@ reporterrs(cfpi)
 	    notaries[notarycnt].not    = notary;
 	    notaries[notarycnt].orcpt  = NULL;
 	    notaries[notarycnt].notify = NULL;
+	    notaries[notarycnt].notifyflgs = 0;
 	    notaries[notarycnt].rcpt   = dnsrecipient;
 	    notaries[notarycnt].rcpntp = rcpntpointer;
 	    if (drptidx > 0) {
@@ -329,10 +327,29 @@ reporterrs(cfpi)
 		  continue;
 		}
 		if (CISTREQN(d,"NOTIFY=",7)) {
-		  notaries[notarycnt].orcpt = d+7;
+		  char *p;
+		  notaries[notarycnt].notify = d+7;
 		  d += 7;
+		  p = d;
 		  while (*d != 0 && (*d != ' ' && *d != '\t')) ++d;
 		  if (*d) *d++ = 0;
+		  while (*p) {
+		    if (CISTREQN(p,"NEVER",5)) {
+		      notaries[notarycnt].notifyflgs |= NOT_NEVER;
+		      p += 5;
+		    } else if (CISTREQN(p,"DELAY",5)) {
+		      notaries[notarycnt].notifyflgs |= NOT_DELAY;
+		      p += 5;
+		    } else if (CISTREQN(p,"SUCCESS",7)) {
+		      notaries[notarycnt].notifyflgs |= NOT_SUCCESS;
+		      p += 7;
+		    } else if (CISTREQN(p,"FAILURE",7)) {
+		      notaries[notarycnt].notifyflgs |= NOT_FAILURE;
+		      p += 7;
+		    } else
+		      break; /* Burp.. Junk! */
+		    if (*p == ',') ++p;
+		  }
 		  continue;
 		}
 		/* XX: Other junk at DSN parameter string ? */
@@ -340,8 +357,7 @@ reporterrs(cfpi)
 		if (*d) *d++ = 0;
 		continue;
 	      }
-	      if (notaries[notarycnt].notify != NULL &&
-		  CISTREQN(notaries[notarycnt].notify,"NEVER",5))
+	      if (notaries[notarycnt].notifyflgs & NOT_NEVER)
 		continue; /* Don't report on this recipient! */
 	    }
 	    ++notarycnt;
@@ -522,6 +538,7 @@ reporterrs(cfpi)
 	free_cfp_memory(cfp);
 }
 
+/*
 char *
 mail_alloc(n)
 	u_int n;
@@ -536,4 +553,4 @@ mail_free(s)
 	free(s);
 	return 0;
 }
-
+*/
