@@ -18,10 +18,11 @@
 #include <errno.h>
 #include "scheduler.h"
 #include <sys/stat.h>
-#include <syslog.h>
+#include "syslog.h"
 #include <fcntl.h>
 #include "mail.h"
 #include <string.h>
+#include "ta.h"
 
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
@@ -125,15 +126,6 @@ static RETSIGTYPE sig_readcf __((int sig));
 extern char *Version;
 extern char *optarg;
 extern char *strerror __((int errno));
-extern int cistrcmp __((char *s1, char *s2));
-extern int efstat __((int fd, struct stat *stbuf));
-extern int emkdir __((char *dirpath, int mode));
-extern int eopen __((char *filename, int flags, int mode));
-extern int eread __((int fd, void *buf, int count));
-extern int errno;
-extern int estat __((char *filename, struct stat *stbuf));
-extern int eunlink __((char *filename));
-extern int optind;
 
 static int timeserver_pid = 0;
 extern time_t mytime __((time_t *));
@@ -229,7 +221,6 @@ struct ctlfile *cfp;
 	if (cfp->contents)	free(cfp->contents);
 	if (cfp->mid)		free(cfp->mid);
 	if (cfp->erroraddr)	free(cfp->erroraddr);
-	if (cfp->errorchan)	free(cfp->errorchan);
 	if (cfp->logident)	free(cfp->logident);
 	if (cfp->envid)		free(cfp->envid);
 	free((char *)cfp);
@@ -449,13 +440,7 @@ main(argc, argv)
 
 	for (i = 0; i < SIZE_L; ++i)
 	  spt_mesh[i] = sp_init();
-#ifdef  LOG_PID
-#ifdef  LOG_MAIL
 	openlog("scheduler", LOG_PID, LOG_MAIL);
-#else   /* !LOG_MAIL */
-	openlog("scheduler", LOG_PID);
-#endif  /* LOG_MAIL */
-#endif  /* LOG_PID */
 	if (optind < argc) {
 	  /* process the specified control files only */
 	  for (; optind < argc; ++optind) {
@@ -1157,7 +1142,8 @@ struct offsort {
 	long	headeroffset;
 	long	drptoffset;
 	char	*sender;
-	char	*dsnrecipient;
+	/* char	*dsnrecipient; */
+	int	notifyflg;
 };
 
 /* ``bcfcn'' is used by the qsort comparison routine,
@@ -1314,8 +1300,8 @@ static struct ctlfile *vtxprep(cfp, file)
 	      while (*cp != 0 && *cp != ' ') ++cp;  *cp = 0; /* We assume: No spaces at the sender addr! */
 	      if (cfp->erroraddr)  free(cfp->erroraddr);
 	      cfp->erroraddr = strsave(latest_sender);
-	      if (cfp->errorchan)  free(cfp->errorchan);
-	      cfp->errorchan = strsave(senderchannel);
+	      if (strcmp(senderchannel,"error")==0)
+		cfp->iserrmesg = 1;
 	      break;
 	    case _CF_RECIPIENT:
 	      if (opcnt >= offspc-1) {
@@ -1334,13 +1320,45 @@ static struct ctlfile *vtxprep(cfp, file)
 	      offarr[opcnt].headeroffset = -1;
 	      offarr[opcnt].drptoffset = -1;
 	      offarr[opcnt].sender = latest_sender;
+	      offarr[opcnt].notifyflg = NOT_FAILURE;
 	      prevrcpt = opcnt;
 	      ++opcnt;
 	      break;
 	    case _CF_RCPTNOTARY:
-	      /* IETF-NOTARY-DRPT DATA! */
-	      if (prevrcpt >= 0)
+	      /* IETF-NOTARY-DRPT+NOTIFY DATA! */
+	      if (prevrcpt >= 0) {
 		offarr[prevrcpt].drptoffset = *lp + 2;
+		cp += 2;
+		/* Lets parse the input, we want NOTIFY= flags */
+		while (*cp != 0) {
+		  while (*cp == ' ' || *cp == '\t') ++cp;
+		  if (CISTREQN("NOTIFY=",cp,7)) {
+		    offarr[opcnt].notifyflg = 0;
+		    cp += 7;
+		    while (*cp) {
+		      if (CISTREQN(cp,"NEVER",5)) {
+			cp += 5;
+			offarr[opcnt].notifyflg |= NOT_NEVER;
+		      } else if (CISTREQN(cp,"DELAY",7)) {
+			cp += 5;
+			offarr[opcnt].notifyflg |= NOT_DELAY;
+		      } else if (CISTREQN(cp,"FAILURE",7)) {
+			cp += 7;
+			offarr[opcnt].notifyflg |= NOT_FAILURE;
+		      } else if (CISTREQN(cp,"SUCCESS",7)) {
+			cp += 7;
+			offarr[opcnt].notifyflg |= NOT_SUCCESS;
+		      } else {
+			break; /* Burp ?? */
+		      }
+		      if (*cp == ',') ++cp;
+		    }
+		  } else {
+		    while (*cp && *cp != ' ' && *cp != '\t') ++cp;
+		  }
+		}
+	      }
+	      prevrcpt = -1; /* Add ONCE! */
 	      break;
 	    case _CF_MSGHEADERS:
 	      for (/* we count up.. */; svn < opcnt; ++svn)
@@ -1576,6 +1594,7 @@ static struct ctlfile *vtxprep(cfp, file)
 	      /* vp->sender       = strsave(offarr[svn].sender); */
 	      vp->headeroffset = offarr[svn].headeroffset; /*They are similar*/
 	      vp->drptoffset   = offarr[svn].drptoffset;
+	      vp->notaryflg    = offarr[svn].notifyflg;
 	      while (svn < i) {
 		vp->index[i-svn-1] = offarr[svn].myidx;
 		++svn;
