@@ -14,11 +14,6 @@
 #include "malloc.h"
 #include "splay.h"
 
-#ifndef	HAVE_SYSLOG
-#define	syslog	(void)
-#endif
-
-
 struct config_entry {
 	struct config_entry *next;
 	char	*channel;	/* channel part of pattern		     */
@@ -73,7 +68,7 @@ struct ctlfile {
 	struct vertex	*head;		/* head of the list of groups	     */
 	int	nlines;		/* number of lines/entries in the file	     */
 	char	*erroraddr;	/* error address(es)			     */
-	char	*errorchan;	/* error channel -- to detect 'MAIL FROM:<>' */
+	int	iserrmesg;	/* error channel -- to detect 'MAIL FROM:<>' */
 	char	*contents;	/* the control file as copied into memory    */
 	char	*logident;	/* identification for log entries	     */
 	char	*deliveryform;	/* [mea] Name of delivery error form	     */
@@ -168,6 +163,11 @@ struct vertex {
 	long		headeroffset;	/* Message headers for this rcpt     */
 	long		drptoffset;	/* IETF-NOTARY DRPT  data	     */
 	char		*notary;	/* IETF Notary report data	     */
+	int		notaryflg;	/* IETF DSN notary control flags     */
+#define NOT_NEVER   001
+#define NOT_DELAY   002
+#define NOT_SUCCESS 004
+#define NOT_FAILURE 010
 	char		*dnsrecipient;	/*				     */
 
 	int		ce_pending;	/* pending on what ?		     */
