@@ -83,8 +83,9 @@ struct threadgroup *thgp;
 if (verbose) printf("delete_threadgroup(%s/%d/%s)\n",
 		    thgp->wchan->name,thgp->withhost,thgp->whost->name);
 
-	if (thgp->idleproc != NULL || thgp->thread != NULL) abort();
-	if (thrg_root == NULL) abort();
+	if (thgp->idleproc != NULL || thgp->thread != NULL)
+	  abort(); /* Deleting non-empty thread-group! */
+	if (thrg_root == NULL) abort(); /* No thread-group root! */
 
 
 	/* Are we last to keep these web links ? */
