@@ -406,7 +406,7 @@ static int ctlowner(cfp)
 	int rc = nope;
 
 	if (cfp->mid == NULL)
-	  abort();
+	  abort(); /* calling-convention error! */
 #ifdef USE_ALLOCA
 	path = alloca(5+strlen(cfp->mid)+sizeof QUEUEDIR+8);
 #else
@@ -681,7 +681,10 @@ static int u_ok(vp, index, inum, offset, notary, message)
 		  message == NULL ? "(sent)" : message);
 	}
 
-	/* XX: Save/process info regarding delivery receipts! */
+	if (vp->notaryflg & NOT_SUCCESS) {
+	  /* Save/process info regarding delivery receipts! */
+	  msgerror(vp, offset, message);
+	}
 
 	if (statuslog)
 	  logstat(statuslog,vp,"ok");
@@ -735,7 +738,8 @@ static int u_error(vp, index, inum, offset, notary, message)
 		"%s %s: %ld/%ld/%s/error %s\n", timestring(),
 		vp->thgrp->ce.command, inum, offset, notary, message);
 
-	msgerror(vp, offset, message);
+	if (vp->notaryflg & NOT_FAILURE)
+	  msgerror(vp, offset, message);
 
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, (off_t)0, 2);
