@@ -81,10 +81,9 @@ extern int wait();
 # include <sys/socket.h>
 # include <netinet/in.h>
 #endif
-#ifdef	HAVE_SYSLOG_H
-# include <syslog.h>
-#endif	/* HAVE_SYSLOG */
-#include <identuser.h>
+
+#include "syslog.h" /* we use local -- we should anyway */
+#include "identuser.h"
 
 #ifndef	SIGCHLD
 #define	SIGCHLD	SIGCLD
@@ -490,15 +489,7 @@ main(argc, argv)
 	      exit(1);
 	    }
 	    setrhostname(&raddr);
-#ifdef LOG_MAIL
-#ifdef  LOG_PID
 	    openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
-#else  /* !LOG_PID */
-	    openlog("smtpserver", 0);
-#endif /* LOG_PID */
-#else
-	    openlog("smtpserver", 0);
-#endif /* LOG_MAIL */
 
 	    if (ident_flag != 0)
 	      setrfc1413ident(0,&raddr);
@@ -507,10 +498,7 @@ main(argc, argv)
 
 	    sprintf(ident_username+strlen(ident_username),
 		    "@%s (port %d)",rhostname, rport);
-#ifdef	LOG_INFO
 	    syslog(LOG_INFO, "connection from %s\n", ident_username);
-#endif /* LOG_INFO */
-
 	    pid = getpid();
 	    if (!logfp)
 	      openlogfp(daemon_flg);
@@ -664,15 +652,7 @@ main(argc, argv)
 		rport = raddr.sin_port;
 		setrhostname(&raddr);
 
-#ifdef  LOG_MAIL
-#ifdef  LOG_PID
 		openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
-#else  /* !LOG_PID */
-		openlog("smtpserver", 0);
-#endif /* LOG_PID */
-#else  /* !LOG_MAIL */
-		openlog("smtpserver", 0);
-#endif /* !LOG_MAIL */
 
 		if (ident_flag != 0)
 		  setrfc1413ident(msgfd,&raddr);
@@ -681,10 +661,7 @@ main(argc, argv)
 
 		sprintf(ident_username+strlen(ident_username),
 			"@%s (port %d)", rhostname, rport);
-#ifdef	LOG_INFO
 		syslog(LOG_INFO, "connection from %s", ident_username);
-#endif /* LOG_INFO */
-
 		pid = getpid();
 
 		if (!logfp)
@@ -825,10 +802,8 @@ reporterr(tell, msg)
 	long tell;
 	char *msg;
 {
-#ifdef	LOG_INFO
 	syslog(LOG_ERR,
 	       "aborted (%ldc) from %s/%d: %s", tell, rhostname, rport, msg);
-#endif	/* LOG_INFO */
 	if (logfp != NULL) {
 	  fprintf(logfp, "%d#\taborted: %s\n", pid, msg);
 	  fflush(logfp);
@@ -1384,7 +1359,7 @@ char *buf, *cp;
 	s = rfc821_path(cp,STYLE(cfinfo, 'R'));
 	if (s == cp) {
 	  /* Failure.. */
-	  type821err(501,"5.5.2",buf,"Path data: %s",rfc821_error);
+	  type821err(501,m552,buf,"Path data: %s",rfc821_error);
 	  return;
 	}
 	if (*s == '>') {
@@ -1631,11 +1606,9 @@ smtp_data(buf,cp)
 	  mfp = NULL;
 	  type(250, "2.6.0", "Roger");
 	  fflush(stdout);
-#ifdef	LOG_INFO
 	  syslog(LOG_INFO,
 		 "accepted id %d (%ldc) from %s/%d",
 		 inum, tell, rhostname, rport);
-#endif /* LOG_INFO */
 	  if (logfp != NULL) {
 	    fprintf(logfp,
 		    "%d#\t%d: %ld bytes\n",
@@ -1767,11 +1740,9 @@ smtp_bdata(buf,cp)
 	  } else {
 	    mfp = NULL;
 	    type(250, "2.6.0", "Roger, stored %ld bytes into spool", tell);
-#ifdef	LOG_INFO
 	    syslog(LOG_INFO,
 		   "accepted id %d (%ldc) from %s/%d",
 		   inum, tell, rhostname, rport);
-#endif /* LOG_INFO */
 	    if (logfp != NULL) {
 	      fprintf(logfp, "%d#\t%d: %ld bytes\n", pid, inum, tell);
 	      fflush(logfp);
@@ -1830,15 +1801,7 @@ smtpserver(insecure)
 	}
 
 	/* re-opening the log ?? */
-#ifdef	LOG_MAIL
-#ifdef	LOG_PID
 	openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
-#else
-	openlog("smtpserver", 0);
-#endif	/* LOG_PID */
-#else	/* !LOG_MAIL */
-	openlog("smtpserver", 0);
-#endif	/* LOG_MAIL */
 
 #ifdef HAVE_TCPD_H /* TCP-Wrapper code */
 	if (wantconn(s_inputfd, "smtp-receiver") == 0) {
@@ -1964,11 +1927,9 @@ smtpserver(insecure)
 	  *cp++ = c;
 	  if (carp->verb == NULL) {
 	    type(550, m552, "Unknown command '%s'", buf);
-#ifdef	LOG_WARNING
 	    syslog(LOG_WARNING,
 		   "unknown SMTP command '%s' from %s/%d",
 		   buf, rhostname, rport);
-#endif /* LOG_WARNING */
 	    fflush(stdout);
 	    continue;
 	  }
@@ -2360,7 +2321,11 @@ tmalloc(n)
 
 static void
 #ifdef HAVE_STDARG_H
-report(char *cp, ...)
+report(char *cp
+#ifdef __STDC__
+	, ...
+#endif
+	)
 #else
 /* VARARGS */
 report(va_alist)
@@ -2479,9 +2444,7 @@ callr()
 
 	if (routerprog == NULL) {
 	  if ((buf = getzenv("MAILBIN")) == NULL) {
-#ifdef	LOG_ERR
 	    syslog(LOG_ERR, "MAILBIN unspecified in zmailer.conf");
-#endif /* LOG_ERR */
 	    return -1;
 	  }
 	  routerprog = emalloc(strlen(buf)+sizeof "router"+2);
@@ -2620,7 +2583,11 @@ router(function, holdlast, args)
 #ifdef HAVE_VPRINTF
 #ifdef HAVE_STDARG_H
 void
-type(int code, char *status, char *fmt, ...)
+type(int code, char *status, char *fmt
+#ifdef __STDC__
+	, ...
+#endif
+	)
 #else
 /* VARARGS2 */
 void
@@ -2776,18 +2743,18 @@ type821err(code,status,inbuf,msg,s1,s2,s3)
 	void *s1, *s2, *s3; /* XX: Propably not portable.. */
 {
 	char *s = inbuf+3+strlen(status)+1;
+	int maxcnt = 200;
 
 	printf("%03d-%s ", code < 0 ? -code : code,status);
 	if (logfp != NULL)
 	  fprintf(logfp, "%dw\t%03d-%s ", pid, code,status);
-	if (((int)(rfc821_error_ptr - s)) < 200)
-	  while (s < rfc821_error_ptr) {
-	    ++s;
-	    putc(' ',stdout);
-	    if (logfp != NULL)
-	      putc(' ',logfp);
-	  }
-	printf("^\n");
+	while (s < rfc821_error_ptr && --maxcnt >= 0) {
+	  ++s;
+	  putc(' ',stdout);
+	  if (logfp != NULL)
+	    putc(' ',logfp);
+	}
+	printf("^\r\n");
 	if (logfp != NULL)
 	  fprintf(logfp,"^\n");
 	type(code,status,msg,s1,s2,s3);
