@@ -281,7 +281,7 @@ process(dp)
 	fputc('\n', mfp);
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 	  /* If not prohibited, print it! */
-	  if ((rp->dsnflags & _DSN_NOTIFY_NEVER) == 0) {
+	  if (rp->notify == NULL || !CISTREQN(rp->notify,"NEVER",5)) {
 	    fprintf(mfp, "error: %s: %s\n",
 		    rp->addr->host, rp->addr->user);
 	    ++reportcnt;
@@ -319,7 +319,7 @@ process(dp)
 	  static char const *type_rfc   = "RFC822";
 	  static char const *type_local = "X-LOCAL";
 
-	  if ((rp->dsnflags & _DSN_NOTIFY_NEVER) != 0)
+	  if (rp->notify != NULL && CISTREQN(rp->notify,"NEVER",5))
 	    continue;
 	  rcpt = rp->addr->user;
 	  if (strchr(rcpt,'@') != NULL) {
@@ -328,11 +328,11 @@ process(dp)
 	      typetag = type_local;
 	  } else
 	    typetag = type_local;
+	  if (rp->orcpt != NULL)
+	    fprintf(mfp, "Original-Recipient: %s\n",rp->orcpt);
 	  fprintf(mfp, "Final-Recipient: %s; %s\n", typetag, rcpt);
 	  fprintf(mfp, "Action: failed\n");
 	  fprintf(mfp, "Diagnostic-Code: X-LOCAL; 500 (%s)\n", rp->addr->host );
-	  if (rp->orcpt != NULL)
-	    fprintf(mfp, "Original-Rcpt: %s\n",rp->orcpt);
 	  fprintf(mfp, "\n");
 	}
 
@@ -348,8 +348,8 @@ process(dp)
 	/* write the (new) headers with local "Received:"-line.. */
 	writeheaders(rp,mfp,"\n",0,0);
 
-	/* If the DSN RET=NO is in effect, don't copy the msg body! */
-	if ((rp->dsnflags & _DSN_RET_NO) == 0) {
+	/* If the DSN RET=HDRS is in effect, don't copy the msg body! */
+	if (!dp->dsnretmode || CISTREQN(dp->dsnretmode,"FULL",4)) {
 
 	  /* Copy out the rest with somewhat more efficient method */
 	  lastchar = 0;
