@@ -415,6 +415,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    rp->desc = &d;
 	    rp->orcpt = NULL;
 	    rp->notify = NULL;
+	    rp->notifyflgs = 0;
 	    d.recipients = rp;
 	    rp->status = EX_OK;
 	    rp->newmsgheader = NULL;
@@ -427,10 +428,29 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	      while (*s) {
 		while (*s && (*s == ' ' || *s == '\t')) ++s;
 		if (cistrncmp("NOTIFY=",s,7)==0) {
+		  char *p;
 		  s += 7;
-		  prevrp->notify = s;
+		  prevrp->notify = p = s;
 		  while (*s && *s != ' ' && *s != '\t') ++s;
 		  if (*s) *s++ = 0;
+		  prevrp->notifyflgs = 0;
+		  while (*p) {
+		    if (cistrncmp("NEVER",p,5)==0) {
+		      p += 5;
+		      prevrp->notifyflgs |= _DSN_NOTIFY_NEVER;
+		    } else if (cistrncmp("DELAY",p,5)==0) {
+		      p += 5;
+		      prevrp->notifyflgs |= _DSN_NOTIFY_DELAY;
+		    } else if (cistrncmp("SUCCESS",p,7)==0) {
+		      p += 7;
+		      prevrp->notifyflgs |= _DSN_NOTIFY_SUCCESS;
+		    } else if (cistrncmp("FAILURE",p,7)==0) {
+		      p += 7;
+		      prevrp->notifyflgs |= _DSN_NOTIFY_FAILURE;
+		    } else
+		      break; /* Burp !? */
+		    if (*p == ',') ++p;
+		  }
 		  continue;
 		}
 		if (cistrncmp("ORCPT=",s,6)==0) {
@@ -443,6 +463,8 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 		/* XX: BOO! Unknown value! */
 		while (*s && *s != ' ' && *s != '\t') ++s;
 	      }
+	      /* Previous entry added, no more..! */
+	      prevrp = NULL;
 	    }
 	    break;
 	  case _CF_MSGHEADERS:
