@@ -85,7 +85,9 @@ extern int res_mkquery(), res_send(), dn_expand(), dn_skipname();
 #endif	/* IPPORT_SMTP */
 
 extern char errormsg[];
+#ifndef strchr
 extern char *strchr();
+#endif
 
 #ifdef	BIND
 
@@ -167,7 +169,7 @@ getrrtype(host, hbsize, rrtype, cnamelevel)
 	    if (qdcount > 0 && strchr(host, '.') == NULL) {
 	      cp = (msgdata *)&answer + sizeof(HEADER);
 	      if (dn_expand((msgdata *)&answer,
-			    eom, cp, host, hbsize) >= 0) {
+			    eom, cp, (u_char*)host, hbsize) >= 0) {
 		if (host[0] == '\0') {
 		  host[0] = '.';
 		  host[1] = '\0';
