@@ -499,7 +499,7 @@ char **ct_linep;	/* Could be multiline! */
 	  } else if (cistrcmp("boundary",paramname)==0) {
 	    /* Parameter:  boundary="...." */
 	    ct->boundary = parval;
-	  } else if (cistrcmp("name",paramname,5)==0) {
+	  } else if (cistrcmp("name",paramname)==0) {
 	    /* Parameter:  name="...." */
 	    ct->name     = parval;
 	  } else {
