@@ -37,6 +37,11 @@ writeheaders(rp,fp,newline,convertmode,maxwidth)
 
 	if (!msgheaders) return -1;
 
+	/* [Thomas Knott]
+	 * Error-Flag des Filedescriptors fp loeschen (HP-UX)
+	 */
+	clearerr(fp);
+
 	while (*msgheaders && !ferror(fp)) {
 		int linelen = strlen(*msgheaders);
 		if (fwrite(*msgheaders,1,linelen,fp) != linelen)
