@@ -167,6 +167,13 @@ char *MBOXLOCKS_default = ".:"; /* Huh ?? No lockf() nor flock() ??? */
 #define MAILMODE        0600    /* prevent snoopers from looking at mail */
 #define	NONUIDSTR	"6963"	/* X:magic nonsense uid if nobody uid is bad */
 
+/* [Thomas Knott] [mea]
+ * The form for "-S" (sendmail-style-behaviour) enabled
+ * "Return-Receipt-To:" -header processing.
+ * This is a file in $MAILSHARE/FORMSDIR/ -directory
+ */
+#define RETURN_RECEIPT_FORM "return-receipt"
+
 /*
  * The following is stuck in for reference only.  You could add
  * alternate spool directories to the list (they are checked in
@@ -284,6 +291,10 @@ extern char **environ;
 extern int writebuf __((FILE *fp, char *buf, int len, int *lastchp));
 extern int writemimeline __((FILE *fp, char *buf, int len, int *lastchp));
 
+static int do_return_receipt_to = 0;
+static void  return_receipt __((struct ctldesc *dp, char *retrecpaddr));
+static char *find_return_receipt_hdr __((struct rcpt *rp));
+
 extern int  qp_to_8bit   __((struct rcpt *rp));
 extern int  qptext_check __((struct rcpt *rp));
 
@@ -370,7 +381,7 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:gh:l:rRMV8")) != EOF) {
+	while ((c = getopt(argc, argv, "abc:gh:l:rRSMV8")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -391,6 +402,9 @@ main(argc, argv)
 	  case 'R':
 	    dorbiff_always = !dorbiff_always;
 	    break;
+	  case 'S':
+	    do_return_receipt_to = 1;
+	    break;
 	  case 'a':		/* toggle mbox st_atime preservation */
 	    keepatime = !keepatime;
 	    break;
@@ -655,9 +669,9 @@ uid_t uid;
 	  cp = NULL;
 	  break;
 	case L_MAXTRYS:
-	  notaryreport(file,"delayed",
-		       "4.4.5 (Mailbox unlocking fails)",
-		       "x-local; 450 (Mailbox unlocking fails)");
+	  notaryreport(file,"failed",
+		       "5.4.5 (Mailbox unlocking fails)",
+		       "x-local; 550 (Mailbox unlocking fails)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL, "\"%s\" is locked", file);
 	  return 1;
 	case L_NAMELEN:
@@ -692,9 +706,9 @@ uid_t uid;
 	if (!havedotlock) {
 	  char mbuf[256];
 	  sprintf(mbuf, "\"%s\": %s", file, strerror(errno));
-	  notaryreport(file,"delayed",
-		       "4.4.5 (File locking with dotlock failed)",
-		       "x-local; 450 (File locking with dotlock failed)");
+	  notaryreport(file,"failed",
+		       "5.4.5 (File locking with dotlock failed)",
+		       "x-local; 550 (File locking with dotlock failed)");
 	  DIAGNOSTIC(rp, errno == EBUSY ? EX_TEMPFAIL :
 		     (errno == EACCES ? EX_NOPERM : EX_UNAVAILABLE),
 		     "can't dotlock %s", mbuf);
@@ -712,9 +726,9 @@ char *file;
 #ifdef	USE_NFSMBOX
 	if (nfslock(file, LOCK_EX) != 0) {
 	  alarm(0);
-	  notaryreport(file,"delayed",
-		       "4.4.5 (File locking with NFS lock failed)",
-		       "x-local; 450 (File locking with NFS lock failed)");
+	  notaryreport(file,"failed",
+		       "5.4.5 (File locking with NFS lock failed)",
+		       "x-local; 550 (File locking with NFS lock failed)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "nfs lock() can't nfslock \"%s\"", file);
 	  return 1;
@@ -735,9 +749,9 @@ char *file;
 	if (lseek(fdmail, (off_t)0, L_SET) < 0 ||
 	    lockf(fdmail, F_LOCK, 0) < 0) {
 	  alarm(0);
-	  notaryreport(file,"delayed",
-		       "4.4.5 (File locking with  lockf  failed)",
-		       "x-local; 450 (File locking with  lockf  failed)");
+	  notaryreport(file,"failed",
+		       "5.4.5 (File locking with  lockf  failed)",
+		       "x-local; 550 (File locking with  lockf  failed)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "can't lockf() \"%s\"", file);
 	  return 1;
@@ -756,9 +770,9 @@ char *file;
 #ifdef	HAVE_FLOCK
 	if (flock(fdmail, LOCK_EX) < 0) {
 	  alarm(0);
-	  notaryreport(file,"delayed",
-		       "4.4.5 (File locking with  flock  failed)",
-		       "x-local; 450 (File locking with  flock  failed)");
+	  notaryreport(file,"failed",
+		       "5.4.5 (File locking with  flock  failed)",
+		       "x-local; 550 (File locking with  flock  failed)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "can't flock() \"%s\"", file);
 	  return 1;
@@ -785,6 +799,7 @@ deliver(dp, rp, timestring)
 	int hasdir;
 	struct stat st, s2;
 	char *file = NULL, *cp, *at, *plus;
+	char *retrecptaddr;
 	int ismbox = 0, i;
 #if	defined(HAVE_SOCKET)
 	struct biffer *nbp = NULL;
@@ -920,17 +935,13 @@ deliver(dp, rp, timestring)
 
 	    char *mailbox = getzenv("MAILBOX");
 	    notaryreport(rp->addr->user,"failed",
-			 "4.3.5 (System mailbox configuration is wrong, we are in deep trouble..)",
-			 "x-local; 466 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
+			 "5.3.5 (System mailbox configuration is wrong, we are in deep trouble..)",
+			 "x-local; 566 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "System mailbox configuration is wrong!  No such directory (%s)!  Aargh!",maildirs[0]);
-#ifdef  LOG_PID
-#ifdef  LOG_MAIL
+
 	    openlog("scheduler", LOG_PID, LOG_MAIL);
-#else   /* !LOG_MAIL */
-	    openlog("scheduler", LOG_PID);
-#endif  /* LOG_MAIL */
-#endif  /* LOG_PID */
+
 	    if (!mailbox) 
 	      syslog(LOG_ALERT, "ZMailer mailbox can't open local mail folder directory! (Using BUILTIN list of choices)\n");
 	    else
@@ -970,8 +981,8 @@ deliver(dp, rp, timestring)
 	}
 	if (exstat(rp, file, &st, lstat) < 0) {
 	  notaryreport(rp->addr->user,"failed",
-		       "4.2.0 (User's mailbox disappeared, will retry)",
-		       "x-local; 466 (User's mailbox disappeared, will retry)");
+		       "5.2.0 (User's mailbox disappeared, will retry)",
+		       "x-local; 566 (User's mailbox disappeared, will retry)");
 	  DIAGNOSTIC(rp, EX_TEMPFAIL,
 		     "mailbox file \"%s\" disappeared", file);
 	  return;
@@ -1002,7 +1013,7 @@ deliver(dp, rp, timestring)
 	
 	if (st.st_nlink > 1) {
 	  notaryreport(rp->addr->user,"failed",
-		       "4.2.1 (Destination file ambiguous)",
+		       "5.2.1 (Destination file ambiguous)",
 		       "x-local; 500 (Destination file has more than one name..)");
 	  DIAGNOSTIC(rp, EX_UNAVAILABLE,
 		     "too many links to file \"%s\"", file);
@@ -1037,7 +1048,7 @@ deliver(dp, rp, timestring)
 	  else
 	    rp->status = EX_UNAVAILABLE;
 	  notaryreport(file,"failed",
-		       "4.2.1 (File open for append failed)",
+		       "5.2.1 (File open for append failed)",
 		       "x-local; 500 (File open for append failed)");
 	  DIAGNOSTIC(rp, rp->status, fmtbuf, file);
 	  setrootuid(rp);
@@ -1059,9 +1070,9 @@ deliver(dp, rp, timestring)
 	}
 	if (st.st_ino != s2.st_ino || st.st_dev != s2.st_dev ||
 	    s2.st_nlink != 1) {
-	    notaryreport(file,"delayed",
-			 "4.4.5 (Lost race for mailbox file)",
-			 "x-local; 450 (Lost race for mailbox file)");
+	    notaryreport(file,"failed",
+			 "5.4.5 (Lost race for mailbox file)",
+			 "x-local; 550 (Lost race for mailbox file)");
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "lost race for mailbox \"%s\"", file);
 	    close(fdmail);
@@ -1247,6 +1258,14 @@ deliver(dp, rp, timestring)
 	  if (nbp != NULL) /* putmail() has produced a DIAGNOSTIC */
 	    nbp->offset = -1;
 #endif	/* BIFF || RBIFF */
+
+	/* [Thomas Knott]  "Return-Receipt-To:" */
+	if (do_return_receipt_to) {
+	  retrecptaddr = find_return_receipt_hdr(rp);
+	  if (retrecptaddr != NULL)
+	    return_receipt(dp,retrecptaddr);
+	}
+
 	return;
 }
 
@@ -1313,11 +1332,11 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	if (*fromuser == 0 ||
 	    strcmp(rp->addr->link->channel, "error") == 0)
 	  fromuser = "postmaster";
-	if (fprintf(fp, "%s%s  %s",
-		    FROM_,fromuser,timestring) == EOF
+	if (fprintf(fp, "%s%s %s",
+		    FROM_,fromuser,timestring) < 0
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
 	  notaryreport(rp->addr->link->user, "failed",
-		       "4.2.2 (Write to user's mailbox failed)",
+		       "5.2.2 (Write to user's mailbox failed)",
 		       "x-local; 500 (Write to user's mailbox failed)");
 	  DIAGNOSTIC(rp, EX_IOERR,
 		     "header write to \"%s\" failed", file);
@@ -1333,12 +1352,14 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	}
 	lastch = appendlet(dp, rp, fp, file, is_mime);
 
-	if (lastch == EOF || ferror(fp) || fflush(fp) == EOF) {
+	if (lastch < 0 || ferror(fp) || fflush(fp) != 0) {
+	  int err;
       write_failure:
+	  err = errno;
 	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC3(rp, EX_IOERR,
 		      "message write to \"%s\" failed: %s",
-		      file, strerror(errno));
+		      file, strerror(err));
 #ifdef HAVE_FTRUNCATE
 	  /* XX: should I really do this? */
 	  fflush(fp);
@@ -1346,6 +1367,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    ftruncate(FILENO(fp), (off_t)eofindex);
 #endif
 	  fclose(fp);
+	  eofindex = -1;
 	  fp = NULL;
 	  goto time_reset;
 	}
@@ -1668,8 +1690,8 @@ creatembox(rp, filep, uid, gid, pw)
 	/* assert *filep != NULL */
 	if (fd == -1) {
 	  notaryreport(rp->addr->user, "failed",
-		       "4.3.1 (can't create user spool mailbox file)",
-		       "x-local; 466 (can't create user spool mailbox file)");
+		       "5.3.1 (can't create user spool mailbox file)",
+		       "x-local; 566 (can't create user spool mailbox file)");
 	  DIAGNOSTIC(rp, EX_CANTCREAT, "can't create \"%s\"", *filep);
 	}
 	/* otherwise the message was printed by createfile() */
@@ -1695,11 +1717,11 @@ createfile(rp, file, uid)
 	    *cp = '\0';
 	    if (exstat(rp, file, &st, stat) < 0) {
 	      *cp = '/';
-	      sprintf(msg,"x-local; 466 (*INVOCATION BUG* Can't create user spool mailbox file: \"%s\", Directory stat() error: %s)",
+	      sprintf(msg,"x-local; 566 (*INVOCATION BUG* Can't create user spool mailbox file: \"%s\", Directory stat() error: %s)",
 		      file,strerror(errno));
 	      *cp = 0;
 	      notaryreport(rp->addr->user, "failed",
-			   "4.3.5 (Something wrong, bad config ?)", msg);
+			   "5.3.5 (Something wrong, bad config ?)", msg);
 	      DIAGNOSTIC(rp, i, "stat failed on %s", file);
 	      return -2;
 	    }
@@ -1707,8 +1729,8 @@ createfile(rp, file, uid)
 	    if (st.st_mode & 020) { /* group writable? */
 	      if (!setupuidgid(rp, uid, st.st_gid)) {
 		notaryreport(rp->addr->user, "failed",
-			     "4.3.5 (The mailbox configuration is faulty)",
-			     "x-local; 466 (*INVOCATION BUG* The mailbox directory is group writable, but can't change my gid to it)");
+			     "5.3.5 (The mailbox configuration is faulty)",
+			     "x-local; 566 (*INVOCATION BUG* The mailbox directory is group writable, but can't change my gid to it)");
 		DIAGNOSTIC(rp, i,
 			   "failed changing group id to create file in \"%s\"",
 			   file);
@@ -1739,9 +1761,9 @@ createfile(rp, file, uid)
 	else /* if (saverrno == ENOENT) */
 	  i = EX_CANTCREAT;
 	/* convoluted to maintain 4 arguments to DIAGNOSTIC */
-	sprintf(msg, "x-local; 466 (error [%s] creating \"%%s\")", strerror(saverrno));
+	sprintf(msg, "x-local; 566 (error [%s] creating \"%%s\")", strerror(saverrno));
 	notaryreport(rp->addr->user, "failed",
-		     "4.3.0 (Other mailsystem error)",
+		     "5.3.0 (Other mailsystem error)",
 		     msg);
 	sprintf(msg, "error (%s) creating \"%%s\"", strerror(saverrno));
 	DIAGNOSTIC(rp, i, msg, file);
@@ -1755,8 +1777,8 @@ createfile(rp, file, uid)
 int
 setupuidgid(rp, uid, gid)
 	struct rcpt *rp;
-	uid_t uid;
-	gid_t gid;
+	int uid;
+	int gid;
 {
 	setrootuid(rp);
 	if (setgid(gid) < 0) {
@@ -1876,7 +1898,7 @@ appendlet(dp, rp, fp, file, ismime)
 	      if (writemimeline(fp, s0, linelen, &lastch) != linelen) {
 		DIAGNOSTIC(rp, EX_IOERR,
 			   "write to \"%s\" failed", file);
-		return EOF;
+		return -1;
 	      }
 	      s0 = s;
 	    }
@@ -1904,14 +1926,14 @@ appendlet(dp, rp, fp, file, ismime)
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
  	      MFPCLOSE;
-	      return EOF;
+	      return -1;
 	    }
 	  }
 	  if (i == EOF && !feof(mfp)) {
 	    DIAGNOSTIC(rp, EX_IOERR,
 		       "read error from message file", 0);
 	    MFPCLOSE;
-	    return EOF;
+	    return -1;
 	  }
 
 	  MFPCLOSE;
@@ -1923,7 +1945,7 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (writebuf(fp, buffer, readalready, &lastch) != readalready) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return EOF;
+	      return -1;
 	    }
 	    rp->status = EX_OK;
 	    return lastch;
@@ -1936,12 +1958,12 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (i < 0) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "read error from message file", 0);
-	      return EOF;
+	      return -1;
 	    }
 	    if (writebuf(fp, buffer, i, &lastch) != i) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return EOF;
+	      return -1;
 	    }
 	    readalready = i;
 	    bufferfull++;
@@ -1964,7 +1986,7 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (writemimeline(fp, s, i, &lastch) != i) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return EOF;
+	      return -1;
 	    }
 	    s = s2;
 	  }
@@ -1973,7 +1995,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  if (writebuf(fp, s, dp->let_end - s, &lastch) != (dp->let_end - s)) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return EOF;
+	      return -1;
 	  }
 	}
 #endif
@@ -2362,3 +2384,142 @@ qptext_check(rp)
 	if (!is_mime) cte = 0;
 	return cte;
 }
+
+static char *
+find_return_receipt_hdr (rp)
+    struct rcpt *rp;
+{
+    char **ptr, *hdr;
+
+    for (ptr = *(rp->newmsgheader); *ptr != NULL; ptr++) {
+        if (strncasecmp(*ptr, "Return-Receipt-To:", 18) == 0)
+            break;
+    }
+
+    if (*ptr == NULL)
+        return (NULL);
+
+    hdr = *ptr + 18;
+    while (*hdr != '\0' && isspace(*hdr))
+        hdr++;
+
+    return(hdr);
+}
+
+static void
+return_receipt (dp, retrecptaddr)
+	struct ctldesc *dp;
+	char *retrecptaddr;
+{
+	char buf[BUFSIZ];
+	char **cpp, *mailshare, *mfpath;
+	FILE *mfp, *efp;
+	struct rcpt *rp;
+	int n;
+	char lastchar;
+	char *boundarystr = buildboundary("return-receipt");
+
+	char *dfltform[] = {
+		"From: The Post Office <postmaster>",
+		"Subject: Returned mail: Return receipt",
+		"MIME-Version: 1.0",
+		"Content-Type: multipart/report; report-type=delivery-status;",
+		"",
+		"Your mail message has been delivered properly to the following recipients:",
+		NULL
+	};
+
+	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
+	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
+	    DIAGNOSTIC(rp, EX_TEMPFAIL, "mail_open failure", 0);
+	  warning("Cannot open mail file!");
+	  return;
+	}
+	fprintf(mfp, "channel error\n");
+
+	rp = dp->recipients;
+
+	/* copy To: from return-receipt address */
+	fprintf(mfp, "To: %s\n", retrecptaddr);
+
+	/* copy error message file itself */
+	if ((mailshare = getzenv("MAILSHARE")) == NULL)
+	  mailshare = MAILSHARE;
+
+	mfpath = emalloc(3+strlen(mailshare)+strlen(FORMSDIR)
+				 +strlen(RETURN_RECEIPT_FORM));
+	sprintf(mfpath, "%s/%s/%s", mailshare, FORMSDIR, RETURN_RECEIPT_FORM);
+
+	if ((efp = fopen(mfpath, "r")) != NULL) {
+	  int inhdr = 1;
+	  buf[sizeof(buf)-1] = 0;
+	  while (fgets(buf,sizeof(buf)-1,efp) != NULL) {
+	    if (strncmp(buf,"HDR",3)==0) {
+	      fputs(buf+4,mfp);
+	    } else if (strncmp(buf,"SUB",3)==0) {
+	      fputs(buf+4,mfp);
+	    } else {
+	      if (inhdr) {
+		inhdr = 0;
+		fprintf(mfp,"MIME-Version: 1.0\n");
+		fprintf(mfp,"Content-Type: multipart/report; report-type=delivery-status;\n");
+		fprintf(mfp,"\tboundary=\"%s\"\n\n\n",boundarystr);
+		fprintf(mfp, "--%s\n", boundarystr);
+		fprintf(mfp, "Content-Type: text/plain\n");
+	      }
+	      fputs(buf,mfp);
+	    }
+	  } /* ... while() ends.. */
+	  fclose(efp);
+	} else {
+	  for (cpp = &dfltform[0]; *cpp != NULL; ++cpp)
+	    if (*cpp == 0) {
+	      fprintf(mfp, "\tboundary=\"%s\"\n\n\n",boundarystr);
+	      fprintf(mfp, "--%s\n", boundarystr);
+	      fprintf(mfp, "Content-Type: text/plain\n");
+	    } else
+	      fprintf(mfp, "%s\n", *cpp);
+	}
+	/* print out errors in standard format */
+	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
+	  fprintf(mfp, "\t%s\n", rp->addr->user);
+	}
+	fprintf(mfp, "\n--%s\n", boundarystr);
+	fprintf(mfp, "Content-Type: message/delivery-status\n\n");
+
+	/* XX: print out errors in IETF-NOTARY format as well! */
+	fprintf(mfp, "Original-MTS-Type: INET\n");
+	if (dp->envid != NULL)
+	  fprintf(mfp, "Original-Envelope-Id: %s\n",dp->envid);
+	fprintf(mfp, "Final-MTS-Type: INET\n");
+	fprintf(mfp, "Final-MTA: %s\n\n", mydomain() );
+
+	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
+	  fprintf(mfp, "Rcpt: %s\n", rp->addr->user);
+	  fprintf(mfp, "Action: delivered\n");
+	  fprintf(mfp, "Status: 250 (%s)\n", rp->addr->host );
+	  if (rp->orcpt != NULL)
+	    fprintf(mfp, "Original-Rcpt: %s\n",rp->orcpt);
+	  fprintf(mfp, "\n");
+	}
+
+	fprintf(mfp, "--%s\n", boundarystr);
+	fprintf(mfp, "Content-Type: message/rfc822\n\n");
+
+	/* XX: Skip over the delivery envelope lines! */
+
+	rp = dp->recipients;
+	/* write the (new) headers with local "Received:"-line.. */
+	writeheaders(rp,mfp,"\n",0,0);
+
+	fprintf(mfp, "--%s--\n", boundarystr);
+	if (ferror(mfp)) {
+	  mail_abort(mfp);
+	  n = EX_IOERR;
+	} else if (mail_close(mfp) == EOF)
+	  n = EX_IOERR;
+	else
+	  n = EX_OK;
+	for (rp = dp->recipients; rp != NULL; rp = rp->next)
+	  DIAGNOSTIC(rp, n, (char *)NULL, 0);
+  }
