@@ -107,7 +107,8 @@ struct maildesc {
 #define	MO_REMOTEFROM		 02000
 #define	MO_CRLF			 04000
 #define MO_BSMTP		010000 /* BSMTP-wrapping -- with HIDDENDOT.. */
-#define MO_BESMTP		020000 /* Extended BSMTP -- SIZE+8BITMIME+? */
+#define MO_BESMTP		020000 /* Extended BSMTP -- SIZE+8BITMIME    */
+#define MO_BEDSMTP		040000 /* EBSMTP + DSN */
 
 struct exmapinfo {
 	int	origstatus;
@@ -468,6 +469,9 @@ deliver(dp, mp, startrp, endrp, verboselog)
 
 	free((char *)av);
 	/* ... having forked and set up the pipe, we quickly continue */
+
+	/* XX: BSMTP et.al. envelope formation here! */
+
 	if (mp->flags & (MO_UNIXFROM|MO_REMOTEFROM)) {
 	  char *timestring;
 	  time_t now;
@@ -615,8 +619,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 			 "x-local; 250 Delivered");
 	  else
 	    notaryreport(rp->addr->user, "failed",
-			 "4.3.0",
-			 "x-local; 430 Unknown error");
+			 "5.3.0",
+			 "x-local; 530 Unknown error");
 	  diagnostic(rp, i, 0, "%s", buf);
 	}
 	/* XX: still need to deal with MO_STRIPQUOTES */
@@ -1054,7 +1058,11 @@ readsmcf(file, mailer)
 	  case 'r':	m.flags |= MO_RFROMFLAG;	break;
 	  case 's':	m.flags |= MO_STRIPQUOTES;	break;
 	  case 'b':	m.flags |= MO_BSMTP;		break;
-	  case 'B':	m.flags |= MO_BESMTP|MO_BSMTP;	break;
+	  case 'B':	if (m.flags & MO_BESMTP) /* -BB */
+			    m.flags |= MO_BEDSMTP;
+			else
+			    m.flags |= MO_BESMTP|MO_BSMTP;
+			break;
 	  case 'A':		/* arpanet-compatibility */
 	  case 'C':		/* canonicalize remote hostnames */
 	  case 'D':		/* this mailer wants a Date: line */
