@@ -888,7 +888,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  for (rp = startrp; rp && rp != endrp; rp = rp->next) {
 	    /* NOTARY: addres / action / status / diagnostic */
 	    notaryreport(rp->addr->user,notaryacct(r,"*NOTOK*"),
-			 "4.5.0 (Undetermined protocol error)",NULL);
+			 "5.5.0 (Undetermined protocol error)",NULL);
 	    diagnostic(rp, r, 0, "%s", remotemsg);
 	  }
 	  if (smtp_isopen)
@@ -1020,8 +1020,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    if (rp->status == EX_OK) {
 	      /* NOTARY: addres / action / status / diagnostic / wtt */
 	      notaryreport(rp->addr->user,notaryacct(EX_TEMPFAIL,"*NOTOK*"),
-			   "4.4.2 (Message write timed out)",
-			   "smtp; 466 (Message write timed out)"); /* XX: FIX THE STATUS? */
+			   "5.4.2 (Message write timed out)",
+			   "smtp; 566 (Message write timed out)"); /* XX: FIX THE STATUS? */
 	      diagnostic(rp, EX_TEMPFAIL, 0, "%s", "header write error");
 	    }
 	  if (verboselog)
@@ -1043,8 +1043,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  for (rp = startrp; rp && rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user, notaryacct(r,"*NOTOK*"),
-			   "4.4.2 (Message write timed out;2)",
-			   "smtp; 466 (Message write timed out;2)"); /* XX: FIX THE STATUS? */
+			   "5.4.2 (Message write timed out;2)",
+			   "smtp; 566 (Message write timed out;2)"); /* XX: FIX THE STATUS? */
 	      diagnostic(rp, r, 0, "%s", remotemsg);
 	    }
 	  /* The failure occurred during processing and was due to an I/O
@@ -1084,13 +1084,16 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 		    logtag(), size, hsize, nrcpt,
 		    body_start - env_start, body_end - body_start);
 	}
-	for (rp = startrp; rp && rp != endrp; rp = rp->next)
+	for (rp = startrp; rp && rp != endrp; rp = rp->next) {
 	  if (rp->status == EX_OK) {
-	    notaryreport(rp->addr->user,
-			 notaryacct(r,"relayed"), /* XXX: NOTARY DRPT ?? */
-			 NULL,NULL);
-	    diagnostic(rp, r, 0, "%s", "" /* remotemsg */);
+	    char *reldel = "-";
+	    if (!(ehlo_capabilities & ESMTP_DSN) &&
+		(rp->notifyflgs & _DSN_NOTIFY_SUCCESS))
+	      reldel = "relayed";
+	    notaryreport(rp->addr->user, reldel, NULL, NULL);
+	    diagnostic(rp, r, 0, "%s", remotemsg);
 	  }
+	}
 
 	/* More recipients to send ? */
 	if (r == EX_OK && more_rp != NULL)
@@ -1339,7 +1342,7 @@ writebuf(fp, buf, len)
 	    state = 0;
 	    if (c == '.') {
 	      if (putc(c, fp) == EOF || putc(c, fp) == EOF) {
-		notaryreport(NULL,FAILED,"4.4.2 (body write error, 1)",
+		notaryreport(NULL,FAILED,"5.4.2 (body write error, 1)",
 			     "smtp; 500 (body write error, 1)");
 		strcpy(remotemsg, "write error 1");
 		return EOF;
@@ -1347,7 +1350,7 @@ writebuf(fp, buf, len)
 	      VLFPRINTF((verboselog,".."));
 	    } else {
 	      if (putc(c, fp) == EOF) {
-		notaryreport(NULL,FAILED,"4.4.2 (body write error, 2)",
+		notaryreport(NULL,FAILED,"5.4.2 (body write error, 2)",
 			     "smtp; 500 (body write error, 2)");
 		strcpy(remotemsg, "write error 2");
 		return EOF;
@@ -1356,7 +1359,7 @@ writebuf(fp, buf, len)
 	    }
 	  } else if (c == '\n') {
 	    if (putc('\r', fp) == EOF || putc(c, fp) == EOF) {
-	      notaryreport(NULL,FAILED,"4.4.2 (body write error, 3)",
+	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 3)",
 			   "smtp; 500 (body write error, 3)");
 	      strcpy(remotemsg, "write error 3");
 	      return EOF;
@@ -1365,7 +1368,7 @@ writebuf(fp, buf, len)
 	    state = 1;
 	  } else {
 	    if (putc(c, fp) == EOF) {
-	      notaryreport(NULL,FAILED,"4.4.2 (body write error, 4)",
+	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 4)",
 			   "smtp; 500 (body write error, 4)");
 	      strcpy(remotemsg, "write error 4");
 	      return EOF;
@@ -1492,7 +1495,7 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 
 	  if (column == 0 && c == '.') {
 	    if (putc(c, fp) == EOF) {
-	      notaryreport(NULL,FAILED,"4.4.2 (body write error, 5)",
+	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 5)",
 			   "smtp; 500 (body write error, 5)");
 	      strcpy(remotemsg, "write error 5");
 	      return EOF;
@@ -1502,7 +1505,7 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 
 	  if (c == '\n') {
 	    if (putc('\r', fp) == EOF) {
-	      notaryreport(NULL,FAILED,"4.4.2 (body write error, 6)",
+	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 6)",
 			   "smtp; 500 (body write error, 6)");
 	      strcpy(remotemsg, "write error 6");
 	      return EOF;
@@ -1511,7 +1514,7 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 	    column = -1;
 	  }
 	  if (putc(c, fp) == EOF) {
-	      notaryreport(NULL,FAILED,"4.4.2 (body write error, 7)",
+	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 7)",
 			   "smtp; 500 (body write error, 7)");
 	    strcpy(remotemsg, "write error 7");
 	    return EOF;
@@ -1713,8 +1716,8 @@ smtpconn(host, fparr, verboselog)
 		|| h_errno == TRY_AGAIN
 #endif /* BIND */
 		) {
-	      sprintf(remotemsg,"smtp; 466 (gethostbyname(%s): try later)",host);
-	      notaryreport(NULL,FAILED,"4.4.3 (dns lookup timeout)",remotemsg);
+	      sprintf(remotemsg,"smtp; 566 (gethostbyname(%s): try later)",host);
+	      notaryreport(NULL,FAILED,"5.4.3 (dns lookup timeout)",remotemsg);
 	      if (verboselog)
 		fprintf(verboselog,"%s\n",remotemsg+6);
 	      return EX_TEMPFAIL;
@@ -1729,7 +1732,7 @@ smtpconn(host, fparr, verboselog)
 	      host = hbuf;
 	    } else if (h_errno == TRY_AGAIN) {
 	      sprintf(remotemsg, "smtp; 466 (Canonicalize, try later: %s)", host);
-	      notaryreport(NULL,FAILED,"4.4.3 (dns lookup 'try again')",remotemsg);
+	      notaryreport(NULL,FAILED,"5.4.3 (dns lookup 'try again')",remotemsg);
 	      if (verboselog)
 		fprintf(verboselog,"%s\n",remotemsg+6);
 	      return EX_TEMPFAIL;
@@ -1738,7 +1741,7 @@ smtpconn(host, fparr, verboselog)
 		 even though CNAME lookups fail (see .UK) */
 #else  /* !BIND */
 	    sprintf(remotemsg, "smtp; 466 (object lookup failed on %s)", host);
-	    notaryreport(NULL,FAILED,"4.4.3 (unspecified dns error)", remotemsg);
+	    notaryreport(NULL,FAILED,"5.4.3 (unspecified dns error)", remotemsg);
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n", remotemsg+6);
 	    /* temporary failure since the router knew it */
@@ -1788,8 +1791,8 @@ smtpconn(host, fparr, verboselog)
 		  || h_errno == TRY_AGAIN
 #endif /* BIND */
 		  ) {
-		sprintf(remotemsg,"smtp; 466 (gethostbyname<%s>: try later)",host);
-		notaryreport(NULL,FAILED,"4.4.3 (dns lookup 'try again')", remotemsg);
+		sprintf(remotemsg,"smtp; 566 (gethostbyname<%s>: try later)",host);
+		notaryreport(NULL,FAILED,"5.4.3 (dns lookup 'try again')", remotemsg);
 		if (verboselog)
 		  fprintf(verboselog,"%s\n",remotemsg+6);
 		return EX_TEMPFAIL;
@@ -1803,7 +1806,7 @@ smtpconn(host, fparr, verboselog)
 		sprintf(remotemsg,
 			"smtp; 500 (nameserver data inconsistency. No MX, no address: '%s', errno=%s)",
 			host,strerror(errno));
-	      notaryreport(NULL,FAILED,"4.4.4 (nameserver data inconsistency)",
+	      notaryreport(NULL,FAILED,"5.4.4 (nameserver data inconsistency)",
 			   remotemsg);
 	      if (verboselog)
 		fprintf(verboselog,"%s\n",remotemsg+6);
@@ -1884,7 +1887,7 @@ makeconn(hp, fdp, verboselog, fparr)
 	    getrr(hostbuf, sizeof hostbuf, (u_short)T_WKS, 2) != 1) {
 	  sprintf(remotemsg,"smtp; 550 (WKS checks: no SMTP capability for host %s)",
 		  hostbuf);
-	  notaryreport(NULL,FAILED,"4.4.4 (WKS Checks: no SMTP capability)", remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.4 (WKS Checks: no SMTP capability)", remotemsg);
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg+6);
 	  return EX_UNAVAILABLE;
@@ -1932,7 +1935,7 @@ fdopen_fail:
 		err = errno;
 		printf("smtp: Failed to fdopen() a socket stream, errno=%d, err='%s' Hmm ??\n",err, strerror(err));
 		fflush(stdout);
-		abort();
+		abort(); /* sock-stream fdopen() failure! */
 		if (fparr[0] != NULL) fclose(fparr[0]); fparr[0] = NULL;
 		if (fparr[1] != NULL) fclose(fparr[1]); fparr[1] = NULL;
 	      }
@@ -1998,7 +2001,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	  se = strerror(errno);
 	  sprintf(remotemsg, "smtp; 500 (Internal error, socket: %s)", se);
-	  notaryreport(NULL,FAILED,"4.4.0 (internal error)",remotemsg);
+	  notaryreport(NULL,FAILED,"5.4.0 (internal error)",remotemsg);
 	  if (conndebug)
 	    fprintf(stderr, "%s\n", remotemsg+6);
 	  if (verboselog)
@@ -2015,7 +2018,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	    if (errno != EADDRINUSE && errno != EADDRNOTAVAIL) {
 	      char *s = strerror(errno);
 	      sprintf(remotemsg, "smtp; 500 (Internal error, bind: %s)", s);
-	      notaryreport(NULL,FAILED,"4.4.0 (internal error, bind)",remotemsg);
+	      notaryreport(NULL,FAILED,"5.4.0 (internal error, bind)",remotemsg);
 	      if (verboselog)
 		fprintf(verboselog,"%s\n", remotemsg+6);
 	      if (conndebug)
@@ -2027,7 +2030,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	  }
 	  if (p < (u_short)(IPPORT_RESERVED/2)) {
 	    sprintf(remotemsg, "too many busy ports");
-	    notaryreport(NULL,FAILED,"4.4.0 (internal error, too many busy ports)","smtp; 500 (Internal error, too many busy ports)");
+	    notaryreport(NULL,FAILED,"5.4.0 (internal error, too many busy ports)","smtp; 500 (Internal error, too many busy ports)");
 	    if (conndebug)
 	      fprintf(stderr, "%s\n", remotemsg+6);
 	    if (verboselog)
@@ -3379,6 +3382,7 @@ void
 notarystatsave(smtpline,status)
 char *smtpline, *status;
 {
+	char statbuf[10];
 	int len = strlen(smtpline)+8+6;
 #ifdef USE_ALLOCA
 	char *str = alloca(len);
@@ -3387,6 +3391,7 @@ char *smtpline, *status;
 #endif
 	char *s = str;
 
+	*statbuf = 0;
 	strcpy(s,"smtp; ");
 	s += 6;
 
@@ -3395,6 +3400,22 @@ char *smtpline, *status;
 	*s++ = *smtpline++;
 	*s++ = ' ';
 	if (len >= 11) {
+	  if (ESMTP_ENHSTATUS & ehlo_capabilities) {
+	    char *p = statbuf;
+	    status = statbuf;
+	    while ((p - statbuf) < sizeof(statbuf)-1) {
+	      u_char c = *smtpline;
+	      if (('0' <= c && c <= '9') || c == '.')
+		*p++ = c;
+	      else
+		break;
+	      ++smtpline;
+	    }
+	    *p = 0;
+	    while (*smtpline == ' ' || *smtpline == '\t')
+	      ++smtpline;
+	  }
+
 	  *s++ = '(';
 	  while (*smtpline) {
 	    switch (*smtpline) {
